import argparse
from .VersionNumber import VersionNumber

class CommandLineParser:
    def __init__(self, arguments, validator):
        parser = argparse.ArgumentParser(description="Parse input command line")
        parser.add_argument('--current-version', nargs=1, help='Current version (upgrade is made from)')
        parser.add_argument('--target-version', nargs=1, help='Target version (upgrade is made to)')
        parser.add_argument('--rootfs', nargs=1, help='Path to target rootfs')
        parser.add_argument('--rootfs-src', nargs=1, default='/', help='Path to source rootfs')
        parser.add_argument('--rootfs-default', nargs=1, default='/home/spectracom/default', help='Path to default rootfs')
        parser.add_argument('--patches-path', nargs=1, help='Path to upgrade patches')
        parser.add_argument('--clean', action='store_true', help='Clean upgrade')
        self.__params = parser.parse_args(arguments)

        if not validator(self.__params):
            raise AttributeError("Base arguments are missed")

    def current_version(self):
        return self.__params.current_version[0]

    def target_version(self):
        return self.__params.target_version[0]

    def rootfs_path(self):
        return self.__params.rootfs[0]

    def rootfs_src_path(self):
        return self.__params.rootfs_src[0]

    def rootfs_dflt_path(self):
        return self.__params.rootfs_default[0]

    def patches_path(self):
        return self.__params.patches_path[0]

    def is_upgrade(self):
        # TODO maybe should be more complete to account for extra information
        vn = VersionNumber()

        current = vn.get_version(self.__params.current_version[0])
        target = vn.get_version(self.__params.target_version[0])

        if current.major == target.major:
            if current.minor == target.minor:
                return current.patch < target.patch
            return current.minor < target.minor
        return current.major < target.major

    def is_clean(self):
        return self.__params.clean
