import os
import re
import subprocess
from .VersionNumber import VersionNumber


class SystemService:

    @staticmethod
    def get_next_version(version, patches_path):
        with open(patches_path + '/' + version + '/next', 'r') as file:
            return file.read().strip()

    @staticmethod
    def get_transition_versions(current_version, target_version, patches_path):
        transition_versions = []
        vn = VersionNumber()

        # strip out potential rc|alpha|beta suffices from versions
        current_version = vn.get_version(current_version).core_version
        target_version = vn.get_version(target_version).core_version

        while current_version != target_version:
            transition_versions.append(current_version)
            current_version = SystemService.get_next_version(current_version, patches_path)

        return transition_versions

    @staticmethod
    def __get_version_patches(version, patches_path, regex):
        version_folder = patches_path + '/' + version
        files = os.listdir(version_folder)
        patches = list(filter(lambda file: regex.match(file), files))
        return sorted([version_folder + '/' + patch for patch in patches])

    @staticmethod
    def __set_clean_flag():
        os.system("touch /tmp/upgrade_clean_flag")

    @staticmethod
    def __is_clean_required(version, patches_path):
        return os.path.isfile(os.path.join(patches_path, version, 'clean'))

    @staticmethod
    def get_version_permanent_patches(version, patches_path):
        return SystemService.__get_version_patches(version, patches_path, re.compile(r'^Permanent_[0-9][0-9]'))

    @staticmethod
    def get_version_upgrade_patches(version, patches_path):
        return SystemService.__get_version_patches(version, patches_path, re.compile(r'^Upgrade_[0-9][0-9]'))

    @staticmethod
    def set_required_clean_flag(current_version, target_version, patches_path):
        versions = SystemService.get_transition_versions(current_version, target_version, patches_path)
        for version in versions:
            if SystemService.__is_clean_required(version, patches_path):
                SystemService.__set_clean_flag()
                return

    @staticmethod
    def get_patches(func_get_version_patches, current_version, target_version, patches_path):
        patches = []
        versions = SystemService.get_transition_versions(current_version, target_version, patches_path)
        for version in versions:
            patches.extend(func_get_version_patches(version, patches_path))
        return patches

    @staticmethod
    def get_permanent_patches(current_version, target_version, patches_path):
        get_patches = SystemService.get_version_permanent_patches
        return SystemService.get_patches(get_patches, current_version, target_version, patches_path)

    @staticmethod
    def get_upgrade_patches(current_version, target_version, patches_path):
        get_patches = SystemService.get_version_upgrade_patches
        return SystemService.get_patches(get_patches, current_version, target_version, patches_path)

    @staticmethod
    def apply_patch(patch, rootfs, board_type, rootfs_src, rootfs_dflt):
        if subprocess.call([patch, '--rootfs', rootfs, "--board-type", board_type, "--rootfs-src", rootfs_src, "--rootfs-default", rootfs_dflt]) != 0:
            raise Exception("ERROR: Patch failed: {0}".format(patch))
